﻿using NMemory;
using NMemory.Indexes;
using NMemory.Tables;

namespace TestProject;

/// <summary>
/// 프로그램
/// </summary>
class Program
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Private

    #region 프로그램 시작하기 - Main()

    /// <summary>
    /// 프로그램 시작하기
    /// </summary>
    private static void Main()
    {
        Database database = new Database();

        Table<Group , int> groupTable  = database.Tables.Create<Group , int>(group  => group.ID , null);
        Table<Member, int> memberTable = database.Tables.Create<Member, int>(member => member.ID, null);

        IIndex<Member, int?> memberGroupIDIndex = memberTable.CreateIndex(new RedBlackTreeIndexFactory(), member => member.GroupID);

        database.Tables.CreateRelation
        (
            groupTable.PrimaryKeyIndex,
            memberGroupIDIndex,
            x => x ?? -1,
            x => x
        );

        groupTable.Insert(new Group() { ID = 1, Name = "그룹 1"});
        groupTable.Insert(new Group() { ID = 2, Name = "그룹 2"});

        memberTable.Insert(new Member() { ID = 1, Name = "김철수", GroupID = 1 });
        memberTable.Insert(new Member() { ID = 2, Name = "홍길동", GroupID = 1 });
        memberTable.Insert(new Member() { ID = 3, Name = "홍삼수", GroupID = 1 });
        memberTable.Insert(new Member() { ID = 4, Name = "김영희", GroupID = 2 });
        memberTable.Insert(new Member() { ID = 5, Name = "이주희", GroupID = 2 });

        memberTable.Update(new Member() { ID = 3, Name = "홍삼수 (수정)", GroupID = 1 });

        foreach(Member member in memberTable)
        {
            Console.WriteLine($"{member.Name}, {member.GroupID}");
        }
    }

    #endregion
}